(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/AbsynLogic.sig,v
   Comment: Analysing Absyn models with BDDs
   Release: scope-0-10

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   AbsynLogic.sig,v 1.7 2002/10/24 20:17:09 wasowski Exp
*)

signature AbsynLogic = sig

	val build     : Absyn.state  -> bdd.bdd
	val build_sys : Absyn.system -> bdd.bdd

	val restrict_to : bdd.bdd -> Absyn.stateid list -> bdd.bdd

	val force_pos : bdd.bdd -> Absyn.stateid list -> bdd.bdd
	val force_neg : bdd.bdd -> Absyn.stateid list -> bdd.bdd

	val force : bdd.bdd -> Absyn.stateid list * Absyn.stateid list -> bdd.bdd

	val nextsat : bdd.bdd -> Absyn.stateid list -> 
		(Absyn.stateid list * Absyn.stateid list) list * bdd.bdd 

	val allsat : bdd.bdd -> Absyn.stateid list ->
		(Absyn.stateid list * Absyn.stateid list) list

	val done : unit -> unit

	val sprMapping  : unit -> string
	val sprSolution : Absyn.stateid list * Absyn.stateid list -> string

(*	   [done] performs memory cleanup (bdd.done, etc()) 
	   Not strictly necessery to call. You may call it to force
	   earlier memory release. Otherwise everything is finilised before
	   next init automatically. 
	   
	   [build s] builds BDD representing legal state configuration of
	   statecharts below given state (legal does not mean reachable!)

	   [build sys] builds BDD representing legal state configuration of
	   the whole system.
	 
	   [restrict_to r ss] return bdd which reflects formula over variables given 
	   in ss (these vars will be free).  It is equivalent to
	   existentially qualifying all variables not in ss, but present in
	   bdd. restrict_to uses makeset so it is safe to give a list of
	   states which contains states not in the variable pool, duplicates
	   etc.

	   [nextsat r] returns a list of positive variables in given
	   arbitrary SAT solution, then a list of negative variables in the
	   same solution and bdd r1 representing the original formula excluding
	   the solution. The next call to nextsat r1 will return another
	   assignment, which is not the case with onesat. 
   
*)

end
