(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/Exp.sml,v
   Comment: Expression representation 
   Release: scope-0-10 

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Representation of expression seems to be the same throughout the project
   (the same syntax is used on input as on output, especially when targetting
   C). Theis has been factored out from Absyn to allow beter division between
   modeuls based on abstract syntax and modules based on lower level
   representations. Expressions will have common shape in all stateges though.

   Exp.sml,v 1.2 2002/10/24 20:17:10 wasowski Exp
*)

structure Exp = struct

  (* some interface *)

  open Prim

  type symbol = Symbol.symbol
  
  (* variables *)
  
  type var = symbol

  (* outputs aka function names *)

  type output  = symbol
  
  (* constants *)

  datatype const = CstI of int | CstR of real | CstC of char	

  (* accesses *)

  datatype access = AccessVar of var		(* x       *)
		  | AccessArr of var * exp	(* x[expr] *)

  (* expressions *)		     

  and exp = Cst    of const
	  | Access of access
	  | Prim1  of unop * exp
	  | Prim2  of binop * exp * exp
	  | Call   of oexp	    (* name, [param] *)

  (* output expression *)

  withtype oexp = output * exp list	    (* name, [param] *)	
  
  (* bexp and exp are only introduced to follow the paper more closely *)

  type bexp = exp
  type aexp = exp
  
  (* assignment *)	       
  
  type assgn = access * aexp

end
